/*
 * define our application and pull in ngRoute and ngAnimate
 */
var app = angular.module('app', ['ngRoute', 'ngAnimate']);

/**
 * App configuration
 */
//app.version = 1;                                // Useless at the moment
app.debug = false;                                // Set to true for develop purposes
app.api_version = 'v1_0';                       // Set the API version for the app

app.api_version = app.debug ? 'debug' : app.api_version;
app.api_domain = app.debug ? 'test.mijn.conditioncheck.nl' : 'mijn.conditioncheck.nl';



/**
 * Directives
 */

// werkt niet??
/*
app.directive('backImg', function(){
    return function(scope, element, attrs){
        var url = attrs.backImg;
        element.css({
            'background-image': 'url(' + url +')',
            'background-size' : 'cover'
        });
    };
});​
*/

/*
 * Route change loader / resolveLoader
 */
app.directive('resolveLoader', function($rootScope, $timeout) {
    return {
        restrict: 'E',
        replace: true,
        template: '<div class="resolveLoader ng-hide"><div class="loader"><img src="img/standard-loader.gif" alt="" /></div></div>',
        link: function(scope, element) {
            $rootScope.$on('$routeChangeStart', function(event, currentRoute, previousRoute) {
                element.removeClass('ng-hide');

                var body = $("html, body");
                body.stop().animate({scrollTop:0}, '500', 'swing', function() {});
            });

            $rootScope.$on('$viewContentLoaded', function() {
                $timeout(function() {
                    element.addClass('ng-hide');

                    setTimeout(function() {
                        var views = $('.ng-leave');
                        for (var i=0; i<views.length; i++) {
                            views[i].remove();
                        }
                    }, 500);
                });
            });
        }
    };
});



/**
 * Custom filters
 */
app.filter('split', function() {
    return function (input, splitChar) {
        return input.split(splitChar);
    }
});



/**
 * Factories
 */
app.factory('vehicleService', function () {
    var vehicles = {};

    return {
        set:function (data) {
            vehicles = data;
        },
        get:function (key) {
            return key ? vehicles[key] : vehicles;
        },
        getEditableFields:function (key) {
            var car = vehicles[key];
            var carFields = car["carFields"];

            var editableFields = [];
            for (var i=0; i<Object.keys(carFields).length; i++) {
                var index = Object.keys(car["carFields"])[i];

                if (carFields[index].editable_in_app == 1)
                    editableFields.push(carFields[index]);
            }

            return editableFields;
        }
    };
});

app.factory('authService', function() {
    var user_id = false;

    var setLocalUser = function(userID) {
        localStorage.user_id = userID;
    };

    var getLocalUser = function() {
        return localStorage.user_id;
    };

    var removeLocalUser = function() {
        localStorage.clear();
    };

    return {
        set:function (userID) {
            user_id = userID;
            setLocalUser(userID);
        },
        getUserId:function () {
            return user_id;
        },
        checkIfIsLocalUser:function () {
            return getLocalUser();
        },
        logout:function () {
            user_id = false;
            setLocalUser(false);
            removeLocalUser();
            $('#backgroundBar').removeClass('animateToTop');

            var vehicleOverview = $('.vehicle-overview');

            vehicleOverview.addClass('ng-leave').addClass('ng-leave-active');

            setTimeout(function() {
                vehicleOverview.addClass('hidden');

                window.location = '#/';
            }, 600);
        }
    }

});

app.factory('maintenanceTurnService', function() {
    var globalComment = '';
    var editedFields = {};

    var cameraPositionImages = {};
    var cameraPositionFileURIs = {};

    return {
        setGlobalComment:function (value) {
            globalComment = value;
        },
        getGlobalComment:function () {
            return globalComment;
        },
        setEditedFields:function (object) {
            editedFields = object;
        },
        getEditedFields:function () {
            return editedFields;
        },

        setCameraPositionImage:function(key, value) {
            cameraPositionImages[key] = value;
        },
        getCameraPositionImages:function() {
            return cameraPositionImages;
        },
        deleteCameraPositionImage:function(key) {
            delete cameraPositionImages[key];
        },
        resetCameraPositionImages:function() {
            cameraPositionImages = {};
        },

        setCameraPositionFileURI:function(key, value) {
            cameraPositionFileURIs[key] = value;
        },
        getCameraPositionFileURIs:function() {
            return cameraPositionFileURIs;
        },
        deleteCameraPositionFileURI:function(key) {
            delete cameraPositionFileURIs[key];
        },
        resetCameraPositionFileURIs:function() {
            cameraPositionFileURIs = {};
        }
    }

});

app.factory('signOutTurnService', function() {
    var globalComment = '';
    var cameraCaptures = {};
    var cameraCaptureFileURIs = {};

    return {
        // Global comment
        setGlobalComment:function (value) {
            globalComment = value;
        },
        getGlobalComment:function () {
            return globalComment;
        },

        // Camera captures
        setCameraCapture:function (key, value) {
            cameraCaptures[key] = value;
        },
        getCameraCaptures:function () {
            return cameraCaptures;
        },
        resetCameraCaptures:function() {
            cameraCaptures = {};
        },
        deleteCameraCapture:function(key) {
            delete cameraCaptures[key];
        },

        // Camera capture file URI's
        setCameraCaptureFileURI:function (key, value) {
            cameraCaptureFileURIs[key] = value;
        },
        getCameraCaptureFileURIs:function () {
            return cameraCaptureFileURIs;
        },
        deleteCameraCaptureFileURI:function(key) {
            delete cameraCaptureFileURIs[key];
        },
        resetCameraCaptureFileURIs:function () {
            cameraCaptureFileURIs = {};
        }
    }

});

app.factory('cameraCaptureService', ['maintenanceTurnService', 'signOutTurnService', function (maintenanceTurnService, signOutTurnService) {
    var cameraOptions = {
        // quality: 50,
        quality: 80,
        correctOrientation: true,
        destinationType: Camera.DestinationType.FILE_URI,
        encodingType: Camera.EncodingType.PNG,
        targetWidth: 1600,
        targetHeight: 1200 /*,
        allowEdit: true */
    };

    return {
        takePicture:function (target, onLoadStart, callback) {

            /*if (confirm("Edit Modus aanzetten?")) {
                cameraOptions.allowEdit = true;
            } else {
                cameraOptions.allowEdit = false;
            }
            if (confirm("JPEG?")) {
                cameraOptions.encodingType = Camera.EncodingType.JPEG;
            } else {
                cameraOptions.encodingType = Camera.EncodingType.PNG;
            }
            if (confirm("Super grote foto?")) {
                cameraOptions.targetWidth = 1600;
                cameraOptions.targetHeight = 1200;
            } else {
                cameraOptions.targetWidth = 1200;
                cameraOptions.targetHeight = 800;
            }*/



            navigator.camera.getPicture(
                function (FILE_URI) {

                    var cameraPosition = $(target);
                    var cameraPositionID = cameraPosition.attr('id');

                    var originalCameraPosition = cameraPosition;

                    var options = new FileUploadOptions();
                    options.fileKey = "file";
                    options.fileName = (new Date()).getTime() + Math.round(Math.random() * 100000000) + '_' + FILE_URI.substr(FILE_URI.lastIndexOf('/') + 1);
                    options.mimeType = "image/jpeg";

                    if (cameraPositionID == 'extra') {
                        cameraPositionID = 'extra_' + new Date().getTime();

                        if (originalCameraPosition.hasClass('turn-out-camera-capture'))
                            signOutTurnService.setCameraCapture(cameraPositionID, options.fileName);
                        else
                            maintenanceTurnService.setCameraPositionImage(cameraPositionID, options.fileName);
                    } else {
                        maintenanceTurnService.setCameraPositionImage(cameraPositionID, options.fileName);
                    }

                    onLoadStart();

                    cameraPosition = $('#'+cameraPositionID);

                    var cameraPositionImageOverlay = cameraPosition.find('.image-state-overlay');
                    cameraPositionImageOverlay.removeClass('error').css('opacity', 1);
                    cameraPositionImageOverlay.find('span').text('Uploaden...');

                    var onFileUploadSuccess = function () {
                        if (originalCameraPosition.hasClass('turn-out-camera-capture'))
                            signOutTurnService.setCameraCaptureFileURI(cameraPositionID, FILE_URI);
                        else
                            maintenanceTurnService.setCameraPositionFileURI(cameraPositionID, FILE_URI);

                        // Upload success
                        var cameraPositionImage = cameraPosition.find('.image-holder .image');
                        cameraPositionImage[0].style.backgroundImage = "url('" + FILE_URI + "')";

                        cameraPositionImageOverlay.removeClass('error').find('span').text('Foto succesvol geupload!');
                        cameraPositionImageOverlay.css('opacity', 1);
                        setTimeout(function () {
                            cameraPositionImageOverlay.css('opacity', 0);
                        }, 2000);

                        callback();
                    };

                    var onFileUploadFail = function (error) {
                        if (originalCameraPosition.hasClass('turn-out-camera-capture')) {
                            signOutTurnService.deleteCameraCaptureFileURI(cameraPositionID);
                            signOutTurnService.deleteCameraCapture(cameraPositionID);
                        } else {
                            maintenanceTurnService.deleteCameraCapture(cameraPositionID);
                            maintenanceTurnService.deleteCameraPositionImage(cameraPositionID);
                        }

                        requestErrorCallback();

                        cameraPositionImageOverlay.addClass('error').find('span').text('Er is een Fout opgetreden bij het uploaden van een foto.');
                        cameraPositionImageOverlay.css('opacity', 1);
                        setTimeout(function () {
                            cameraPositionImageOverlay.css('opacity', 0);
                        }, 4000);
                    };

                    var ft = new FileTransfer();

                    ft.onprogress = function (progressEvent) {
                        var statusOverlay = cameraPositionImageOverlay.find('span');

                        if (progressEvent.lengthComputable) {
                            var percentage = Math.floor(progressEvent.loaded / progressEvent.total * 100);
                            statusOverlay.text(percentage + "% geupload...");
                        } else {
                            if (statusOverlay.text() == "" || statusOverlay.text() == "Uploaden...")
                                statusOverlay.text("Loading");
                            else
                                statusOverlay.innerHTML += ".";
                        }
                    };

                    if (originalCameraPosition.hasClass('turn-out-camera-capture'))
                        ft.upload(FILE_URI, encodeURI('http://'+app.api_domain+'/api/'+app.api_version+'/upload-turn-out-image'), onFileUploadSuccess, onFileUploadFail, options);
                    else
                        ft.upload(FILE_URI, encodeURI('http://'+app.api_domain+'/api/'+app.api_version+'/upload-camera-image'), onFileUploadSuccess, onFileUploadFail, options);

                },
                function (error) {
                    //alert('Camera capture error: ' + JSON.stringify(error));
                },
                cameraOptions
            );
        }
    }
}]);



/**
 * Directives
 */
app.directive('takePictureEvent',
    ['cameraCaptureService',
    function(cameraCaptureService) {
        return {
            restrict: 'C',
            link: function($scope, $element, $attrs) {
                $element.on('click', function($event) {
                    cameraCaptureService.takePicture(
                        $event.currentTarget,
                        function() { $scope.$apply(); },
                        function() { $scope.$apply(); }
                    );
                });
            }
        }
    }
]);



/**
 * Global functions
 */
/*
 * Global HTTP error callback
 */
function requestErrorCallback(error) {
    var message = 'Er is iets fout gegaan';

    if( !navigator.onLine )
        message = 'U heeft geen internetverbinding';

    showError(message);
}

function showError(message) {
    $('.resolveLoader').addClass('ng-hide');

    $('.error-message')
        .text(message)
        .css('top', '0px');

    setTimeout(function() {
        $('.error-message').css('top', '-100px');
    }, 3500);
}



/**
 * Routes
 */
var pagesDirectory = 'pages/';
app.config(function($routeProvider) {

    $routeProvider
        .when('/', {
            templateUrl: pagesDirectory + 'login.html',
            controller: 'loginController',
            resolve: {
                "checkLocalUser":function(authService, $location) {
                    if(authService.checkIfIsLocalUser()) {
                        authService.set( authService.checkIfIsLocalUser() );

                        var loginScreen = $('.login');
                        loginScreen.css('display', 'none');
                        window.location = '#/vehicle-overview';
                        $('#backgroundBar').addClass('animateToTop');
                    }
                }
            }
        })
        .when('/vehicle-overview', {
            templateUrl: pagesDirectory + 'vehicle-overview.html',
            controller: 'vehicleOverviewController'
        })
        .when('/vehicle-check/:carId?', {
            templateUrl: pagesDirectory + 'vehicle-check.html',
            controller: 'vehicleCheckController'
        })
        .when('/vehicle-properties/:carId?', {
            templateUrl: pagesDirectory + 'vehicle-properties.html',
            controller: 'vehiclePropertiesController'
        })
        .when('/sign-check/:carId?', {
            templateUrl: pagesDirectory + 'sign-check.html',
            controller: 'signCheckController'
        })
        .when('/vehicle-sign-out/:carId?', {
            templateUrl: pagesDirectory + 'vehicle-sign-out.html',
            controller: 'VehicleSignOutController'
        })

});





/**
 * ----------------------------
 * Controllers
 * ----------------------------
 */

/**
 * loginController
 * --------------------------------------------------------------
 */
app.controller('loginController', function($scope, $http, authService) {
    $http.defaults.useXDomain=true;
    delete $http.defaults.headers.common['X-Requested-With'];

    $scope.pageClass = 'login';

    $scope.data = {
        email: app.debug ? '' : '',
        password: app.debug ? '' : ''
    };

    /*
     * Login API call
     */
    $scope.login = function() {
        $('input').blur();

        $http({
            method: 'POST',
            url: 'http://'+app.api_domain+'/api/'+app.api_version+'/login',
            data: {
                email: $scope.data.email,
                password: $scope.data.password
            }
        }).then(function successCallback(response) {
            if(response.data == 'error') {
                /*
                 * Auth failed...
                 */
                $('.error-message')
                    .text('Foutieve email en wachtwoord combinatie')
                    .css('top', '0px');

                setTimeout(function() {
                    $('.error-message').css('top', '-100px');
                }, 3000);
            } else if(response.data == 'inactive') {
                /*
                 * Auth failed...
                 */
                $('.error-message')
                    .text('Uw Condition Check is inactief')
                    .css('top', '0px');

                setTimeout(function() {
                    $('.error-message').css('top', '-100px');
                }, 3000);
            } else {
                /*
                 * Auth successful!
                 */
                var loginScreen = $('.login');

                loginScreen.addClass('ng-leave').addClass('ng-leave-active');

                $('#backgroundBar').addClass('animateToTop');

                setTimeout(function() {
                    loginScreen.addClass('hidden');

                    window.location = '#/vehicle-overview'
                }, 600);

                authService.set( response.data.id );
            }
        }, requestErrorCallback);
    };

});

/**
 * vehicleOverviewController
 * --------------------------------------------------------------
 */
app.controller('vehicleOverviewController',function($scope, $http, $timeout, vehicleService, maintenanceTurnService, signOutTurnService, authService) {
    $http.defaults.useXDomain=true;
    delete $http.defaults.headers.common['X-Requested-With'];

    $scope.pageClass = 'vehicle-overview';

    $scope.data = {
        vehicles: {},
        searchInput: '',
        maintenanceImages: []
    };

    $scope.fullscreenMode = false;
    $scope.selectedImage = '';
    $scope.selectedImageTitle = '';
    $scope.imagesLoading = false;

    var _timeout = false;

    $scope.logout = function() {
        authService.logout();
    };

    $scope.toggleFullscreen = function(imageData) {

        $scope.fullscreenMode = true;

        var $zoomWrapper = $('.full-image-popup .zoomWrapper');

        $zoomWrapper.find('img').width('');
        $zoomWrapper.find('img').height('');

        var refreshZoom = function () {

            $zoomWrapper.panzoom("reset");
            $zoomWrapper.panzoom("destroy");

            var img = $(this);
            $timeout(function () {

                console.log("Image dimensions: ", $(img).width(), $(img).height(), $(img).attr('src'));

                var img_width = $(img).width();
                var img_height = $(img).height();

                // alert('Img: ' + img_width+ ' / ' + img_height);

                var win_width = window.innerWidth;
                var win_height = window.innerHeight;

                // alert('Win: ' + win_width + ' / ' + win_height);

                if (win_width / img_width < win_height / img_height) {
                    $(img).height(win_height);
                } else {
                    $(img).width(win_width);
                }

                // alert('Height: ' + win_height);
                // alert('Width: '+ win_width);


                var panX = -1 * ($(img).width() - win_width) / 2;
                var panY = -1 * ($(img).height() - win_height) / 2;

                // $zoomWrapper.css('marginTop', win_height / 2);

                $zoomWrapper.width($(img).width());
                $zoomWrapper.height($(img).height());
                $zoomWrapper.panzoom({
                    disablePan: false,
                    minScale: 1,
                    maxScale: 4.0,
                    increment: 0.5,
                    rangeStep: 0.15,
                    contain: 'invert',
                    startTransform: 'scale(.9)',
                });
                // $zoomWrapper.panzoom("zoom", (window.innerWidth / img_width), {animate: true});
                $zoomWrapper.panzoom("reset");
                $zoomWrapper.panzoom("zoom", 1);
                $('.zoomWrapper').panzoom("pan", panX, panY, {animate: true});


            });

        }

        /*if ($scope.selectedImageTitle == imageData.position) {
            alert("Same Image");
            refreshZoom();
        } else {*/
            $zoomWrapper.find('img').attr('src', '');
            $zoomWrapper.find('img').off().on('load', refreshZoom);
            $scope.selectedImage = imageData.img;
            $scope.selectedImageTitle = imageData.position;
        // }

        $timeout(function () {
            var $zoomWrapper = $('.full-image-popup .zoomWrapper');

            try {
                // $zoomWrapper.panzoom({
                //     disablePan: false,
                //     minScale: 1.0,
                //     maxScale: 4.0,
                //     contain: 'invert'
                // });
                // $zoomWrapper.panzoom("reset");

            } catch (err) {
               alert(JSON.stringify(err, null, 2));
            }

        });
    };

    $scope.closeFullscreen = function(imageData) {
        $scope.fullscreenMode = false;
        $scope.selectedImage = '';
        $scope.selectedImageTitle = '';
        // alert(JSON.stringify($scope.fullscreenMode, null, 2));
    };

    $scope.swipeLeft = function() {
        alert('swipe left');
    };

    $scope.startVehicleProcess = function(carID, isLeased) {
        $scope.data.maintenanceImages = [];

        if (isLeased == 1) {

            $('.button-blue.yes').on('click', function() {
                $('.resolveLoader').removeClass('ng-hide');

                $http({
                    method: 'POST',
                    url: 'http://'+app.api_domain+'/api/'+app.api_version+'/save-correct-sign-out-turn',
                    data: {
                        car_id: carID,
                        user_id: authService.getUserId()
                    }
                }).then(function successCallback(response) {
                    response = JSON.parse(response.data);

                    if (response == 'OK') {
                        var flashMessage = $('.flash-message');
                        flashMessage.text('Het voertuig is succesvol afgemeld.');
                        flashMessage.css('top', '0px');

                        setTimeout(function() {
                            flashMessage.css('top', '-100px');
                        }, 4000);

                        $('.sign-out-popup').fadeOut(200);

                        $scope.search();
                    } else if(response == 'ERROR_MAIL') {
                        showError('Fout bij het versturen van de mail.');
                    } else {
                        requestErrorCallback();
                    }

                    $('.resolveLoader').addClass('ng-hide');
                }, function() {
                    $('.resolveLoader').addClass('ng-hide');

                    requestErrorCallback();
                });
            });

            $('.button-blue.no').on('click', function() {
                $('.resolveLoader').removeClass('ng-hide');

                window.location = '#vehicle-sign-out/'+carID;
            });

            $('.sign-out-popup').fadeIn(200);

            // Load images into popup
            var postData = {
                car_id: carID,
                turnid: 1193,
                user_id: authService.getUserId()
            };
            // alert('before http call' + JSON.stringify(postData, null, 2));
            try {
                $scope.imagesLoading = true;

                $http({
                    method: 'POST',
                    url: 'http://'+app.api_domain+'/api/'+app.api_version+'/maintenanceturn-images',
                    data: postData
                }).success(function(response) {

                    $scope.imagesLoading = false;
                    try {
                        // alert(JSON.stringify(response));

                        // response = JSON.parse(response.data);
                        // var images = response.images;
                        // alert(JSON.stringify(images, null, 2));

                        $scope.data.maintenanceImages = response.images;

                        // alert(JSON.stringify($scope.data.maintenanceImages, null, 2));


                        $timeout(function () {
                            // alert('panzoom? ' + JSON.stringify($.fn.panzoom, null, 2));

                            // $('.slideshowWrap .imageWrap').panzoom({
                            //     disablePan: true,
                            //     minScale: 1.0,
                            //     maxScale: 4.0
                            // });

                            // $('.slideshowWrap').panzoom({
                            //     disablePan: true,
                            //     minScale: 1.0,
                            //     maxScale: 4.0
                            // });
                        });

                    } catch (err) {
                        alert(JSON.stringify(err, null, 2));
                    }

                })
                .error(function(data, status, headers, config) {
                    alert('call failed: ' + JSON.stringify({
                        data: data,
                        status: status,
                        headers: headers,
                        config: config
                    }, null, 2));
                });
            } catch (err) {
                alert(JSON.stringify(err, null, 2));
            }


        } else {

            window.location = '#vehicle-check/'+carID;

        }
    };

    $scope.closeSignOutProcessPopup = function() {
        $('.sign-out-popup').fadeOut(200);
    };

    $scope.search = function() {
        // if there is already a timeout in process cancel it
        if(_timeout) {
            $timeout.cancel(_timeout);
        }

        // Set new timeout
        _timeout = $timeout(function(){
            $http({
                method: 'GET',
                url: 'http://'+app.api_domain+'/api/'+app.api_version+'/get-vehicles',
                params: { input: $scope.searchInput, userid: authService.getUserId() }
            }).then(function successCallback(response) {
                /*
                 * Search successful, get search related data.
                 */
                vehicleService.set(response.data.vehicles);

                $scope.data.vehicles = vehicleService.get();
            }, requestErrorCallback);

            _timeout = null;
        }, _timeout === false ? 0 : 300);
    };

    $scope.$on('$viewContentLoaded', function() {
        $scope.search();
        maintenanceTurnService.resetCameraPositionImages();
        maintenanceTurnService.resetCameraPositionFileURIs();
        signOutTurnService.resetCameraCaptures();
        signOutTurnService.resetCameraCaptureFileURIs();
    });

    $scope.$on('$routeChangeStart', function(next, current) {
        $('input').blur();
    });
});

/**
 * vehicleCheckController
 * --------------------------------------------------------------
 */
app.controller('vehicleCheckController', function($scope, vehicleService, maintenanceTurnService, $routeParams) {
    $scope.pageClass = 'vehicle-check';

    var carId = $routeParams.carId;

    $scope.data = {
        vehicle: vehicleService.get(carId),
        globalComment: maintenanceTurnService.getGlobalComment(),
        editableFields: vehicleService.getEditableFields(carId),
        cameraPositionFileURIs: maintenanceTurnService.getCameraPositionFileURIs(),
        cameraPositionImages: maintenanceTurnService.getCameraPositionImages()
    };

    $scope.deletePicture = function(ID) {
        maintenanceTurnService.deleteCameraPositionFileURI(ID);
        maintenanceTurnService.deleteCameraPositionImage(ID);
        $scope.$apply();
    };

    $scope.$watch('data.globalComment', function(newValue, oldValue) {
        maintenanceTurnService.setGlobalComment( newValue );
    });

    $scope.$on('$routeChangeStart', function(next, current) {
        $('input').blur();
    });

});

/**
 * vehiclePropertiesController
 * --------------------------------------------------------------
 */
app.controller('vehiclePropertiesController', function($scope, vehicleService, maintenanceTurnService, $routeParams) {
    $scope.pageClass = 'vehicle-properties';

    var carId = $routeParams.carId;

    $scope.data = {
        editedFields: {},
        vehicle: vehicleService.get(carId)
    };

    $scope.$on('$locationChangeStart', function() {
        maintenanceTurnService.setEditedFields($scope.data.editedFields);
    });

    $scope.$on('$routeChangeStart', function(next, current) {
        $('input').blur();
    });
});

/**
 * signCheckController
 * --------------------------------------------------------------
 */
app.controller('signCheckController', function($scope, $http, authService, vehicleService, maintenanceTurnService, $routeParams) {
    $http.defaults.useXDomain=true;
    delete $http.defaults.headers.common['X-Requested-With'];

    $scope.pageClass = 'sign-check';

    $scope.isLoading = false;

    var carId = $routeParams.carId;
    var signaturePad;

    $scope.data = {
        name_renter: '',
        email_renter: '',
        vehicle: vehicleService.get(carId),
        editableFields: vehicleService.getEditableFields(carId)
    };

    /*
     * Initialize jSignature
     */
    $scope.$on('$viewContentLoaded', function() {
        var wrapper = document.getElementById("signature-pad"),
            clearButton = wrapper.querySelector("[data-action=clear]"),
            canvas = wrapper.querySelector("canvas");

        // Adjust canvas coordinate space taking into account pixel ratio,
        // to make it look crisp on mobile devices.
        // This also causes canvas to be cleared.
        function resizeCanvas() {
            // When zoomed out to less than 100%, for some very strange reason,
            // some browsers report devicePixelRatio as less than 1
            // and only part of the canvas is cleared then.
            var ratio =  Math.max(window.devicePixelRatio || 1, 1);
            canvas.width = canvas.offsetWidth * ratio;
            canvas.height = canvas.offsetHeight * ratio;
            canvas.getContext("2d").scale(ratio, ratio);
        }

        window.onresize = resizeCanvas;
        resizeCanvas();

        signaturePad = new SignaturePad(canvas);

        clearButton.addEventListener("click", function (event) {
            signaturePad.clear();
        });
    });

    /*
     * Submit and save the maintenance turn
     */
    $scope.submitMaintenanceTurn = function() {
        $('.resolveLoader').removeClass('ng-hide');

        var signatureImage;
        if (signaturePad.isEmpty ())
            signatureImage = '';
        else
            signatureImage = signaturePad.toDataURL();

        $scope.isLoading = true;

        //alert(JSON.stringify('is Loading? ' + true, null, 2));

        setTimeout(function() {
            $http({
                method: 'POST',
                url: 'http://'+app.api_domain+'/api/'+app.api_version+'/save-maintenance-turn',
                data: {
                    name_renter: $('#name_renter_input').val(),
                    email_renter: $('#email_renter_input').val(),
                    signature_image: signatureImage,
                    car_id: carId,
                    images: maintenanceTurnService.getCameraPositionImages(),
                    comment: maintenanceTurnService.getGlobalComment(),
                    editedFields: maintenanceTurnService.getEditedFields(),
                    user_id: authService.getUserId()
                }
            }).then(function successCallback(response) {
                response = JSON.parse(response.data);

                if (response == 'OK') {
                    $scope.isLoading = false;

                    window.location = '#/vehicle-overview';

                    var flashMessage = $('.flash-message');
                    flashMessage.text('De check is succesvol opgeslagen.');
                    flashMessage.css('top', '0px');

                    maintenanceTurnService.resetCameraPositionImages();
                    maintenanceTurnService.resetCameraPositionFileURIs();
                    maintenanceTurnService.setGlobalComment('');
                    maintenanceTurnService.editedFields = {};

                    setTimeout(function() {
                        flashMessage.css('top', '-100px');
                    }, 4000);
                } else if(response == 'ERROR_MAIL') {
                    showError('Fout bij het versturen van de mail.');
                } else {
                    requestErrorCallback();
                }

                $scope.isLoading = false;

            }, requestErrorCallback);
        }, 500);
    };

    $scope.$on('$routeChangeStart', function(next, current) {
        $('input').blur();
    });
});

/**
 * VehicleSignOutController
 * --------------------------------------------------------------
 */
app.controller('VehicleSignOutController', function($scope, $http, authService, vehicleService, signOutTurnService, $routeParams) {
    $http.defaults.useXDomain=true;
    delete $http.defaults.headers.common['X-Requested-With'];

    $scope.pageClass = 'vehicle-sign-out';

    var carId = $routeParams.carId;

    $scope.isLoadingSignOut = false;

    $scope.data = {
        vehicle: vehicleService.get(carId),
        globalComment: signOutTurnService.getGlobalComment(),
        cameraCaptures: signOutTurnService.getCameraCaptures(),
        cameraCaptureFileURIs: signOutTurnService.getCameraCaptureFileURIs()
    };

    $scope.deletePicture = function(ID) {
        signOutTurnService.deleteCameraCaptureFileURI(ID);
        signOutTurnService.deleteCameraCapture(ID);
        $scope.$apply();
    };

    $scope.$watch('data.globalComment', function(newValue, oldValue) {
        signOutTurnService.setGlobalComment(newValue);
    });

    $scope.$on('$routeChangeStart', function(next, current) {
        $('input').blur();
    });

    $scope.submitSignOutTurn = function() {

        $scope.isLoadingSignOut = true;

        $('.resolveLoader').removeClass('ng-hide');

        $http({
            method: 'POST',
            url: 'http://'+app.api_domain+'/api/'+app.api_version+'/save-sign-out-turn',
            data: {
                car_id: carId,
                images: signOutTurnService.getCameraCaptures(),
                comment: signOutTurnService.getGlobalComment(),
                user_id: authService.getUserId()
            }
        }).then(function successCallback(response) {
            response = JSON.parse(response.data);

            if (response == 'OK') {

                window.location = '#/vehicle-overview';

                var flashMessage = $('.flash-message');
                flashMessage.text('Het voertuig is succesvol afgemeld.');
                flashMessage.css('top', '0px');

                signOutTurnService.resetCameraCaptures();
                signOutTurnService.resetCameraCaptureFileURIs();
                signOutTurnService.setGlobalComment('');

                setTimeout(function() {
                    flashMessage.css('top', '-100px');
                }, 4000);
            } else if(response == 'ERROR_MAIL') {
                showError('Fout bij het versturen van de mail.');
            } else {
                requestErrorCallback();
            }

            $scope.isLoadingSignOut;
        }, requestErrorCallback);
    };
});

$(function() {
    // $('.slideshowWrap').panzoom();
});